# Define the Win32 and Appx targets for removal
$remove_win32 = "Microsoft Edge", "Microsoft Edge Update", "Microsoft EdgeWebView"
$remove_appx = "MicrosoftEdge", "Win32WebViewHost"

# Set policies to prevent reinstallation
$policies = @('HKLM:\SOFTWARE\Policies', 'HKLM:\SOFTWARE', 'HKLM:\SOFTWARE\WOW6432Node')
foreach ($p in $policies) {
    $path = "$p\Microsoft\EdgeUpdate"
    Set-ItemProperty -Path $path -Name "InstallDefault" -Value 0 -Force > $null 2>&1
    Set-ItemProperty -Path $path -Name "Install{56EB18F8-B008-4CBD-B6D2-8C97FE7E9062}" -Value 0 -Force > $null 2>&1
    Set-ItemProperty -Path $path -Name "Install{F3017226-FE2A-4295-8BDF-00C3A9A7E4C5}" -Value 1 -Force > $null 2>&1
    Set-ItemProperty -Path $path -Name "DoNotUpdateToEdgeWithChromium" -Value 1 -Force > $null 2>&1
}

$edgeupdate = 'Microsoft\EdgeUpdate\Clients\{56EB18F8-B008-4CBD-B6D2-8C97FE7E9062}'
$commands = @("on-logon-autolaunch", "on-logon-startup-boost", "on-os-upgrade")
foreach ($p in @('HKLM:\SOFTWARE', 'HKLM:\SOFTWARE\Wow6432Node')) {
    foreach ($cmd in $commands) {
        $path = "$p\$edgeupdate\Commands\$cmd"
        Set-ItemProperty -Path $path -Name "CommandLine" -Value 'systray.exe' -Force > $null 2>&1
    }
}

# Clear Win32 uninstall block
foreach ($hk in 'HKCU:', 'HKLM:') {
    foreach ($wow in '', '\Wow6432Node') {
        foreach ($i in $remove_win32) {
            $path = "$hk\SOFTWARE$wow\Microsoft\Windows\CurrentVersion\Uninstall\$i"
            Remove-ItemProperty -Path $path -Name NoRemove -Force -ErrorAction SilentlyContinue
            Set-ItemProperty -Path $path -Name AllowUninstall -Value 1 -Force > $null 2>&1
        }
    }
}

# Find all Edge setup.exe and gather BHO paths
$setup = @()
$bho = @("$env:ProgramData\ie_to_edge_stub.exe", "$env:Public\ie_to_edge_stub.exe")

"LocalApplicationData", "ProgramFilesX86", "ProgramFiles" | ForEach-Object {
    $setup += Get-ChildItem "$([Environment]::GetFolderPath($_))\Microsoft\Edge*\setup.exe" -Recurse -ErrorAction SilentlyContinue
    $bho += Get-ChildItem "$([Environment]::GetFolderPath($_))\Microsoft\Edge*\ie_to_edge_stub.exe" -Recurse -ErrorAction SilentlyContinue
}

# Shut down Edge related processes
$processesToKill = 'MicrosoftEdgeUpdate','chredge','msedge','edge','msedgewebview2','Widgets'
foreach ($processName in $processesToKill) {
    Get-Process -Name $processName -ErrorAction SilentlyContinue | ForEach-Object { Stop-Process $_ -Force }
}

# Use dedicated C:\Scripts path for OpenWebSearch innovative redirector
$DIR = "$env:SystemDrive\Scripts"
$null = mkdir $DIR -ErrorAction SilentlyContinue

foreach ($b in $bho) {
    if (Test-Path $b) {
        try {
            Copy-Item $b "$DIR\ie_to_edge_stub.exe" -Force -ErrorAction SilentlyContinue
        } catch { }
    }
}

# Clear Appx uninstall block and remove
$provisioned = Get-AppxProvisionedPackage -Online
$appxpackage = Get-AppxPackage -AllUsers
$store = 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Appx\AppxAllUserStore'
$users = 'S-1-5-18'

if (Test-Path $store) {
    $users += (Get-ChildItem $store | Where-Object { $_ -like '*S-1-5-21*' }).PSChildName
}

foreach ($choice in $remove_appx) {
    if ($choice.Trim()) {
        foreach ($appx in $provisioned | Where-Object { $_.PackageName -like "*$choice*" }) {
            $PackageFamilyName = ($appxpackage | Where-Object { $_.Name -eq $appx.DisplayName }).PackageFamilyName
            $path = "$store\Deprovisioned\$PackageFamilyName"
            New-Item -Path $path -Name '' -Value '' -Force > $null 2>&1
            cmd /c "dism /online /remove-provisionedappxpackage /packagename:$($appx.PackageName) >nul 2>nul"
        }

        foreach ($appx in $appxpackage | Where-Object { $_.PackageFullName -like "*$choice*" }) {
            $PackageFamilyName = $appx.PackageFamilyName
            $PackageFullName = $appx.PackageFullName
            foreach ($app in (Get-ItemProperty "$store\InboxApplications\*$($appx.Name)*").Path.PSChildName) {
                cmd /c "reg delete ""$store\InboxApplications\$app"" /f >nul 2>nul"
            }
            cmd /c "reg add ""$store\Deprovisioned\$PackageFamilyName"" /f >nul 2>nul"
            foreach ($sid in $users) {
                $registryPath = "$store\EndOfLife\$sid\$PackageFullName"
                cmd /c "reg add ""$store\EndOfLife\$sid\$PackageFullName"" /f >nul 2>nul"
            }
            cmd /c "dism /online /set-nonremovableapppolicy /packagefamily:$PackageFamilyName /nonremovable:0 >nul 2>nul"
            powershell -nop -c "remove-appxpackage -package '$PackageFullName' -AllUsers" 2>&1 >''
            foreach ($sid in $users) {
                cmd /c "reg delete ""$store\EndOfLife\$sid\$PackageFullName"" /f >nul 2>nul"
            }
        }
    }
}

# Run Edge setup.exe with uninstall arguments
$purge = '--uninstall --system-level --force-uninstall'
foreach ($s in $setup) {
    try {
        Start-Process -Wait $s -ArgumentList "--msedge $purge"
    } catch { }
}

# Prevent LCU failures by removing EndOfLife Edge entries
foreach ($i in $remove_appx) {
    Get-ChildItem "$store\EndOfLife" -Recurse -ErrorAction SilentlyContinue | Where-Object { $_.Name -like "*${i}*" } | Remove-Item -Force -ErrorAction SilentlyContinue
    Get-ChildItem "$store\Deleted\EndOfLife" -Recurse -ErrorAction SilentlyContinue | Where-Object { $_.Name -like "*${i}*" } | Remove-Item -Force -ErrorAction SilentlyContinue
}

# Extra cleanup
$desktop = [Environment]::GetFolderPath('Desktop')
$appdata = [Environment]::GetFolderPath('ApplicationData')
Remove-Item -Path "$appdata\Microsoft\Internet Explorer\Quick Launch\User Pinned\TaskBar\Tombstones\Microsoft Edge.lnk" -Force -ErrorAction SilentlyContinue
Remove-Item -Path "$appdata\Microsoft\Internet Explorer\Quick Launch\Microsoft Edge.lnk" -Force -ErrorAction SilentlyContinue
Remove-Item -Path "$desktop\Microsoft Edge.lnk" -Force -ErrorAction SilentlyContinue

# Set registry keys for microsoft-edge protocol
$IFEO = 'HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Image File Execution Options'
$MSEP = ("$env:ProgramFiles", "$env:ProgramFiles(x86)")[[Environment]::Is64BitOperatingSystem] + '\Microsoft\Edge\Application'
$MIN = ('--headless', '--width 1 --height 1')[([Environment]::OSVersion.Version.Build) -gt 25179]
$CMD = "$env:systemroot\system32\conhost.exe $MIN"

Set-ItemProperty -Path 'REGISTRY::HKEY_CLASSES_ROOT\microsoft-edge' -Name '(Default)' -Value 'URL:microsoft-edge' -Force
Set-ItemProperty -Path 'REGISTRY::HKEY_CLASSES_ROOT\microsoft-edge' -Name 'URL Protocol' -Value '' -Force
Set-ItemProperty -Path 'REGISTRY::HKEY_CLASSES_ROOT\microsoft-edge' -Name 'NoOpenWith' -Value '' -Force
New-Item -Path 'REGISTRY::HKEY_CLASSES_ROOT\microsoft-edge\shell\open\command' -Name '(Default)' -Value 'explorer.exe' -Force

if (-not (Test-Path "$IFEO\msedge.exe")) {
    New-Item -Path "$IFEO\msedge.exe" -Force
}
Set-ItemProperty -Path "$IFEO\msedge.exe" -Name 'Debugger' -Value $CMD -Force
