## Define targets to remove
$remove_win32 = "Microsoft Edge", "Microsoft Edge Update", "Microsoft EdgeWebView"
$remove_appx = "MicrosoftEdge", "Win32WebViewHost"

## Set useless policies
foreach ($p in 'HKLM:\SOFTWARE\Policies', 'HKLM:\SOFTWARE', 'HKLM:\SOFTWARE\WOW6432Node') {
    $path = "$p\Microsoft\EdgeUpdate"
    Write-Host "Setting useless policies for $path"
    Set-ItemProperty -Path $path -Name "InstallDefault" -Value 0 -Force > $null 2>&1
    Set-ItemProperty -Path $path -Name "Install{56EB18F8-B008-4CBD-B6D2-8C97FE7E9062}" -Value 0 -Force > $null 2>&1
    Set-ItemProperty -Path $path -Name "Install{F3017226-FE2A-4295-8BDF-00C3A9A7E4C5}" -Value 1 -Force > $null 2>&1
    Set-ItemProperty -Path $path -Name "DoNotUpdateToEdgeWithChromium" -Value 1 -Force > $null 2>&1
}
Write-Host "Useless policies set successfully"
Write-Host "---------------------------------"

## Clear Win32 uninstall block
foreach ($hk in 'HKCU:', 'HKLM:') {
    foreach ($wow in '', '\Wow6432Node') {
        foreach ($i in $remove_win32) {
            $path = "$hk\SOFTWARE$wow\Microsoft\Windows\CurrentVersion\Uninstall\$i"
            Remove-ItemProperty -Path $path -Name NoRemove -Force -ErrorAction SilentlyContinue
            Set-ItemProperty -Path $path -Name AllowUninstall -Value 1 -Force > $null 2>&1
        }
    }
}
Write-Host "Win32 uninstall block cleared successfully"
Write-Host "--------------------------------------------"

## Find all Edge setup.exe and gather BHO paths
$setup = ""
$bho = "$env:ProgramData\ie_to_edge_stub.exe","$env:Public\ie_to_edge_stub.exe"

"LocalApplicationData", "ProgramFilesX86", "ProgramFiles" | ForEach-Object {
    $setup += Get-ChildItem "$([Environment]::GetFolderPath($_))\Microsoft\Edge*\setup.exe" -Recurse -ErrorAction SilentlyContinue
    $bho += Get-ChildItem "$([Environment]::GetFolderPath($_))\Microsoft\Edge*\ie_to_edge_stub.exe" -Recurse -ErrorAction SilentlyContinue
}
Write-Host "Edge setup.exe and BHO paths gathered successfully"
Write-Host "---------------------------------------------------"

## Shut down Edge processes
$processesToKill = 'MicrosoftEdgeUpdate', 'chredge', 'msedge', 'edge', 'msedgewebview2', 'Widgets'
foreach ($processName in $processesToKill) {
    Get-Process -Name $processName -ErrorAction SilentlyContinue | ForEach-Object { Stop-Process $_ -Force }
}
Write-Host "Edge processes shut down successfully"
Write-Host "-------------------------------------"

## Use dedicated C:\Scripts path due to Sigma rules FUD
$DIR = "$env:SystemDrive\Scripts"
$null = mkdir $DIR -ErrorAction SilentlyContinue

## Export OpenWebSearch innovative redirector
foreach ($b in $bho) {
    if (Test-Path $b) {
        try {
            Copy-Item $b "$DIR\ie_to_edge_stub.exe" -Force -ErrorAction SilentlyContinue
        } catch { }
    }
}
Write-Host "OpenWebSearch redirector exported successfully"
Write-Host "------------------------------------------------"

## Clear appx uninstall block and remove
$provisioned = Get-AppxProvisionedPackage -Online
$appxpackage = Get-AppxPackage -AllUsers
$store = 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\Appx\AppxAllUserStore'
$store_reg = ($store)

Write-Host "Processing AppX packages"
Write-Host "------------------------"

$users = 'S-1-5-18'

if (Test-Path $store) {
    $users += (Get-ChildItem $store | Where-Object { $_ -like '*S-1-5-21*' }).PSChildName
}

foreach ($choice in $remove_appx) {
    if ('' -eq $choice.Trim()) { continue }

    foreach ($appx in $provisioned | Where-Object { $_.PackageName -like "*$choice*" }) {
        $PackageFamilyName = ($appxpackage | Where-Object { $_.Name -eq $appx.DisplayName }).PackageFamilyName
        Write-Host "Removing provisioned package: $appx.DisplayName"
        $path = "$store_reg\Deprovisioned\$PackageFamilyName"
        New-Item -Path '$store_reg\Deprovisioned\$PackageFamilyName' -Name '' -Value '' -Force
        cmd /c "dism /online /remove-provisionedappxpackage /packagename:$($appx.PackageName) >nul 2>nul"
    }

    foreach ($appx in $appxpackage | Where-Object { $_.PackageFullName -like "*$choice*" }) {
        $inbox = (Get-ItemProperty "$store\InboxApplications\*$($appx.Name)*").Path.PSChildName
        $PackageFamilyName = $appx.PackageFamilyName
        $PackageFullName = $appx.PackageFullName

        foreach ($app in $inbox) {
            Write-Host "Removing inbox application: $app"
            cmd /c "reg delete ""$store_reg\InboxApplications\$app"" /f >nul 2>nul"
        }

        $registryPath = "$store_reg\Deprovisioned\$PackageFamilyName"
        New-Item -Path $registryPath -Force | Out-Null
        cmd /c "reg add ""$store_reg\Deprovisioned\$PackageFamilyName"" /f >nul 2>nul"

        foreach ($sid in $users) {
            $registryPath = "$store_reg\EndOfLife\$sid\$PackageFullName"
            cmd /c "reg add ""$store_reg\EndOfLife\$sid\$PackageFullName"" /f >nul 2>nul"
        }
        cmd /c "dism /online /set-nonremovableapppolicy /packagefamily:$PackageFamilyName /nonremovable:0 >nul 2>nul"
        powershell -nop -c "remove-appxpackage -package '$PackageFullName' -AllUsers" 2>&1 >''
        foreach ($sid in $users) {
            $registryPath = "$store_reg\EndOfLife\$sid\$PackageFullName"
            cmd /c "reg delete ""$store_reg\EndOfLife\$sid\$PackageFullName"" /f >nul 2>nul"
        }
    }
}
Write-Host "AppX packages processed successfully"
Write-Host "--------------------------------------"

## Shut down Edge processes again
foreach ($processName in $processesToKill) {
    Get-Process -Name $processName -ErrorAction SilentlyContinue | ForEach-Object { Stop-Process $_ -Force }
}
Write-Host "Edge processes shut down again successfully"
Write-Host "-------------------------------------------"

## Run Edge setup.exe for uninstall
$purge = '--uninstall --system-level --force-uninstall'
$setup = " Microsoft Edge","Microsoft Edge"

foreach ($s in $setup) {
    try {
        Write-Host "Running $s setup for uninstall"
        Start-Process -Wait $s -ArgumentList " systray.exe $purge" -Force -ErrorAction SilentlyContinue
    } catch {
        # Catch any errors if the process fails
    }
}
Write-Host "Edge setup.exe run for uninstall successfully"
Write-Host "------------------------------------------------"

## Prevent LCU failures due to non-matching EndOfLife Edge entries
foreach ($i in $remove_appx) {
    Get-ChildItem "$store\EndOfLife" -Recurse -ErrorAction SilentlyContinue | Where-Object { $_ -like "*${i}*" } | ForEach-Object {
        $registryKey = $_.Name
        if (Test-Path $registryKey) {
            Remove-Item -Path $registryKey -Force -ErrorAction SilentlyContinue
        }
    }

    Get-ChildItem "$store\Deleted\EndOfLife" -Recurse -ErrorAction SilentlyContinue | Where-Object { $_ -like "*${i}*" } | ForEach-Object {
        $registryKey = $_.Name
        if (Test-Path $registryKey) {
            Remove-Item -Path $registryKey -Force -ErrorAction SilentlyContinue
        }
    }
}
Write-Host "LCU failures prevention completed"
Write-Host "---------------------------------"

## Extra cleanup
$appdata = [Environment]::GetFolderPath('ApplicationData')
$desktop = [Environment]::GetFolderPath('Desktop')

Remove-Item -Path "$appdata\Microsoft\Internet Explorer\Quick Launch\User Pinned\TaskBar\Tombstones\Microsoft Edge.lnk" -Force -ErrorAction SilentlyContinue
Remove-Item -Path "$appdata\Microsoft\Internet Explorer\Quick Launch\Microsoft Edge.lnk" -Force -ErrorAction SilentlyContinue
Remove-Item -Path "$desktop\Microsoft Edge.lnk" -Force -ErrorAction SilentlyContinue
Write-Host "Extra cleanup completed"
Write-Host "-------------------------"

## Set registry keys
$DIR = (Get-Location).Path
Set-ItemProperty -Path 'REGISTRY::HKEY_CLASSES_ROOT\microsoft-edge' -Name '(Default)' -Value 'URL:microsoft-edge' -Force
Set-ItemProperty -Path 'REGISTRY::HKEY_CLASSES_ROOT\microsoft-edge' -Name 'URL Protocol' -Value '' -Force
Set-ItemProperty -Path 'REGISTRY::HKEY_CLASSES_ROOT\microsoft-edge\shell\open\command' -Name '(Default)' -Value "$DIR\ie_to_edge_cmd.exe" -Force
Write-Host "Registry keys set successfully"
Write-Host "--------------------------------"

## Final message
Write-Host "Microsoft Edge removal script executed successfully."
Write-Host "----------------------------------------------------"

## Function to press any key to exit
function PressAnyKeyToExit {
    Write-Host "`nPress any key to exit..."
    $null = $Host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
}

## Call the function to wait for user input
PressAnyKeyToExit
