<#
.SYNOPSIS
Complete uninstaller script for OneDrive on Windows 10 and 11.
.DESCRIPTION
This script completely removes OneDrive, including all related files and registry entries.
.NOTES
Run this script as an administrator.
#>

# Function to display colored messages
function Show-Message {
    param (
        [string]$Message,
        [string]$Color = "White"
    )
    Write-Host $Message -ForegroundColor $Color
}

# Function to check for administrator privileges
function Test-Admin {
    $currentPrincipal = New-Object Security.Principal.WindowsPrincipal([Security.Principal.WindowsIdentity]::GetCurrent())
    if (-not $currentPrincipal.IsInRole([Security.Principal.WindowsBuiltInRole]::Administrator)) {
        Show-Message "########### ERROR - ADMINISTRATOR PRIVILEGES REQUIRED #############" "Red"
        Show-Message "This script must be run as administrator to work properly." "White"
        Show-Message "Right click on the PowerShell icon and select 'Run As Administrator'" "White"
        Exit 1
    }
}

# Function to locate OneDrive setup executable
function Find-OneDriveSetup {
    $paths = @(
        "$env:SystemRoot\SysWOW64\OneDriveSetup.exe",
        "$env:SystemRoot\System32\OneDriveSetup.exe",
        "$env:LOCALAPPDATA\Microsoft\OneDrive\Update\OneDriveSetup.exe",
        "$env:ProgramFiles\Microsoft OneDrive\OneDriveSetup.exe",
        "$env:ProgramFiles(x86)\Microsoft OneDrive\OneDriveSetup.exe"
    )
    
    foreach ($path in $paths) {
        if (Test-Path $path) {
            return $path
        }
    }
    
    Show-Message "OneDriveSetup.exe not found in standard locations." "Red"
    Exit 1
}

# Function to uninstall OneDrive
function Uninstall-OneDrive {
    # Terminate OneDrive process
    Show-Message "Terminating OneDrive process." "Yellow"
    Stop-Process -Name "OneDrive" -Force -ErrorAction SilentlyContinue

    # Find and run OneDriveSetup for uninstallation
    $oneDriveSetup = Find-OneDriveSetup
    if ($oneDriveSetup) {
        Show-Message "Found OneDriveSetup at: $oneDriveSetup" "Green"
        & $oneDriveSetup /uninstall
    } else {
        Show-Message "Unable to find OneDriveSetup executable." "Red"
        Exit 1
    }

    # Remove remaining OneDrive folders
    Show-Message "Removing remaining OneDrive folders." "Yellow"
    Remove-Item -Path "$env:USERPROFILE\OneDrive" -Recurse -Force -ErrorAction SilentlyContinue
    Remove-Item -Path "$env:LOCALAPPDATA\Microsoft\OneDrive" -Recurse -Force -ErrorAction SilentlyContinue
    Remove-Item -Path "$env:ProgramData\Microsoft OneDrive" -Recurse -Force -ErrorAction SilentlyContinue
    Remove-Item -Path "C:\OneDriveTemp" -Recurse -Force -ErrorAction SilentlyContinue
    Remove-Item -Path "$env:APPDATA\Microsoft\Windows\Start Menu\Programs\OneDrive.lnk" -Force -ErrorAction SilentlyContinue

    Show-Message "'The system cannot find the file specified.' errors are ok, it means the files are already gone." "Yellow"

    # Remove OneDrive registry keys
    Show-Message "Removing OneDrive registry keys." "Yellow"
    Remove-ItemProperty -Path "HKCR:\CLSID\{018D5C66-4533-4307-9B53-224DE2ED1FE6}" -Name "System.IsPinnedToNameSpaceTree" -Force -ErrorAction SilentlyContinue
    Remove-Item -Path "HKCR:\Wow6432Node\CLSID\{018D5C66-4533-4307-9B53-224DE2ED1FE6}" -Recurse -Force -ErrorAction SilentlyContinue
    Remove-Item -Path "HKCR:\CLSID\{018D5C66-4533-4307-9B53-224DE2ED1FE6}" -Recurse -Force -ErrorAction SilentlyContinue
}

# Main
Show-Message "------Windows 10/11 OneDriveRemover by RAIYAN------" "Cyan"
Test-Admin

Show-Message "WARNING" "Red"
Show-Message "This script will completely and permanently remove OneDrive from your computer." "White"
Show-Message "Make sure all OneDrive documents that are stored locally are fully backed up before proceeding." "White"

$confirmation = Read-Host "Press 'Y' to continue or any other key to exit"
if ($confirmation -eq 'Y' -or $confirmation -eq 'y') {
    Uninstall-OneDrive
    Show-Message "OneDrive Uninstall and cleaning completed." "Green"
    Show-Message "Successfully OneDrive remove by OneDriveRemover by RAIYAN." "Cyan"
} else {
    Show-Message "Operation cancelled by user." "Red"
}
Pause
